/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

#ifndef LOADER_SDL_H_INCLUDED
#define LOADER_SDL_H_INCLUDED

/* This class (Loader_SDL) implements a Loader for the SDL Optimized
   movie file.  The constructor takes a single argument - the filename.
   NOTE:  All frames passed out of this Loader are SDL_Surfaces, and should
   be type-cast as such!
*/

#include "Loader.h"
#include <stdlib.h>
#include <stdio.h>
#include <SDL/SDL.h>

class Loader_SDL : public Loader{
private:
  FILE *movieFile;
  int headerOffset;
  int w,h,numFrames,frameSize,curFrame;
  Uint8 bitDepth;
  Uint16 pitch;
  Uint32 Rmask,Gmask,Bmask,Amask;
  SDL_Surface *frameSurface;
  unsigned char *frameBuff;

public:
  Loader_SDL(char *filename);

  ~Loader_SDL(){
    if(frameSurface) SDL_FreeSurface(frameSurface);
    if(frameBuff) delete[] frameBuff;
    fclose(movieFile);
  };

  // Returns a buffer containing the current frame.
  virtual unsigned char *getCurFrame(void){
    if(curFrame>0){
      fseek(movieFile,-frameSize,SEEK_CUR);
    }
    fread(frameBuff,1,frameSize,movieFile);
    SDL_FreeSurface(frameSurface);
    frameSurface=SDL_CreateRGBSurfaceFrom(frameBuff,w,h,bitDepth,pitch,Rmask,Gmask,Bmask,Amask);
    return (unsigned char *)(frameSurface);
  };

  // Steps one frame forward and returns a buffer of the new current frame.
  // Returns NULL if there are no more frames.
  virtual unsigned char *getNextFrame(void){
    //    fprintf(stderr,"GetNext\n");
    if(fread(frameBuff,1,frameSize,movieFile)){
      curFrame++;
      SDL_FreeSurface(frameSurface);
      frameSurface=SDL_CreateRGBSurfaceFrom(frameBuff,w,h,bitDepth,pitch,Rmask,Gmask,Bmask,Amask);
      return (unsigned char *)(frameSurface);
    }
    else return NULL;
  };

  // Steps one frame back and returns a buffer of the new current frame.
  // Returns NULL if there are no previous frames.
  virtual unsigned char *getPrevFrame(void){
    if(curFrame>1){
      curFrame--;
      fseek(movieFile,-(2*frameSize),SEEK_CUR);
      fread(frameBuff,1,frameSize,movieFile);
      SDL_FreeSurface(frameSurface);
      frameSurface=SDL_CreateRGBSurfaceFrom(frameBuff,w,h,bitDepth,pitch,Rmask,Gmask,Bmask,Amask);
      return (unsigned char *)(frameSurface);
    }
    else return NULL;
  };

  // Seeks to a particular frame number, given by 'frameNumber'.  Returns
  // an error if the frameNumber is out of range.
  virtual int seekTo(int frameNumber){
    if(frameNumber>=numFrames) return -1;
    curFrame=frameNumber;
    fseek(movieFile,headerOffset+(frameNumber*frameSize),SEEK_SET);
    return 0;
  };

  // Returns the number of frames in the movie
  virtual int length(void){return numFrames;};

  // Returns the width of a frame in the movie, in pixels
  virtual int width(void){return w;};

  // Returns the height of a frame in the movie, in pixels
  virtual int height(void){return h;};

  // Returns the color-depth of a frame in the movie, in bytes
  virtual int depth(void){return 3;};

  // Returns the current frame number.
  virtual int frame(void){return curFrame;};
};

#endif
