#ifndef MOVIE_H_INCLUDED
#define MOVIE_H_INCLUDED

/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

/* This is the header for the Movie class, which encapsulates all the interface
   that is desired for a movie player.  It uses a Loader object to obtain frame
   data, and passes that data to a Renderer object, pursuant to the commands
   that are issued to it.  This class may also be sub-classed to provide custom
   interaction methods (i.e., translation, multi-threading, etc.)  This most
   general version of the Movie class does not know how to create any Loaders
   or Renderers itself, but must rely on the developer to provide them.
*/

#include "Loader/Loader.h"
#include "Renderer/Renderer.h"
#include <stdlib.h>
#include <stdio.h>

#define MOVIE_STEP_FORWARD 0
#define MOVIE_STEP_REVERSE 1

class Movie{
private:
  Loader *loader;
  Renderer *renderer;

  enum State{STOP=0,PLAY=1,FAST_FORWARD=2,
	     FAST_FORWARD_NOPLAY=3,
	     REWIND=4,REWIND_NOPLAY=5};

  State state;

public:
  Movie(void):
    loader(NULL),
    renderer(NULL),
    state(STOP)
  {};

  ~Movie(void){
    if(renderer) delete renderer;
    renderer=NULL;
    if(loader) delete loader;
    renderer=NULL;
  };

  virtual void init(int argc,char *argv[]){};

  virtual void init(Loader *loaderSet,Renderer *rendererSet){
    loaderOpen(loaderSet);
    rendererOpen(rendererSet);
  };

  virtual int play(void){
    state=PLAY;
    return 0;
  };

  virtual int stop(void){
    state=STOP;
    return 0;
  };

  virtual int fastforward(void){
    if(state==PLAY || state==FAST_FORWARD || state==REWIND) state=FAST_FORWARD;
    else state=FAST_FORWARD_NOPLAY;
  };

  virtual int rewind(void){
    if(state==PLAY || state==FAST_FORWARD || state==REWIND) state=REWIND;
    else state=REWIND_NOPLAY;
  };

  virtual int reset(void){
    if(!loader) return -1;
    return loader->seekTo(0);
  };

  virtual int step(int direction=MOVIE_STEP_FORWARD){
    if(!loader || !renderer) return -1;
    if(direction==MOVIE_STEP_FORWARD)
      return renderer->draw(loader->getNextFrame());
    else
      return renderer->draw(loader->getPrevFrame());
  };

  virtual int seek(int frameNumber){
    if(!loader) return -1;
    return loader->seekTo(frameNumber);
  };

  virtual int frame(void){
    if(!loader) return -1;
    return loader->frame();
  };

  virtual int length(void){
    if(!loader) return -1;
    return loader->length();
  };

  virtual void loaderClose(void){
    if(loader) delete loader;
    loader=NULL;
  };

  virtual void loaderOpen(Loader *loaderSet){
    if(loader) loaderClose();
    loader=loaderSet;
  };

  virtual void rendererClose(void){
    if(renderer) delete renderer;
    renderer=NULL;
  };

  virtual void rendererOpen(Renderer *rendererSet){
    if(!loader) return;
    if(renderer) rendererClose();
    renderer=rendererSet;
    renderer->open(loader->width(),loader->height(),loader->depth());
  };

  virtual int update(void){
    if(!loader || !renderer) return 0;
    switch(state){
    case STOP:
      return 0;
      break;
    case PLAY:
      return renderer->draw(loader->getNextFrame());
      break;
    case FAST_FORWARD:
      if(loader->seekTo(loader->frame()+2))
	return -1;
      else
	return renderer->draw(loader->getCurFrame());
      break;
    case FAST_FORWARD_NOPLAY:
      return loader->seekTo(loader->frame()+2);
      break;
    case REWIND:
      if(loader->seekTo(loader->frame()-2))
	return -1;
      else
	return renderer->draw(loader->getCurFrame());
      break;
    case REWIND_NOPLAY:
      return loader->seekTo(loader->frame()-2);
      break;
    };
    return 0;
  };
};

#endif
