/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

/* This is a simple program that takes a series of pnm files and
   produces a FlatFile movie, suitable for play by the various
   Loader_FlatFile* loaders.
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <netinet/in.h>

/* Based on code from Justin Hensley's RGBAImage Class */

char *rgba;
char filename[256];
char filename1[256];
int count;
int numberOfFrames;
int frameOffset;
char filenamePre[128];
char filenamePost[128];
char outFilename[128];
int x,y;

enum RGBAImageFileType
{
	PBM = 0, // Magic Number > P1
	PGM,     // Magic Number > P2
	PPM,     // Magic Number > P3
	RPBM,    // Magic Number > P4
	RPGM,    // Magic Number > P5
	RPPM,    // Magic Number > P6
	PNMA,    // Portable Any Map with Alpha channel. Magic number > P7
	RPNMA    // Raw Portable Any Map with Alpha channel. Magic number > P8
};

inline int discardCommentLine(FILE *ci)
{
    unsigned char current_byte;
    int returnVal=0;
    
    current_byte = fgetc(ci);
//    fprintf(stderr,"\n%c",current_byte);
    if(current_byte == '#' || current_byte=='\n'){ /* comment or blank line */
      if(current_byte=='\n') returnVal=1;
      else{
	while((current_byte = fgetc(ci)) && (current_byte != '\n'));
	returnVal=1;
      };
    }
//	    fprintf(stderr,"\n%c",current_byte);
    else /* oops!, better put it back */
      ungetc(current_byte,ci);

    return returnVal;
}/* End of discardCommentLine() */

int
pnmLoadImage(char *filename,int x,int y,char *rgb)
{
    int depth, index;
    int r, g, b, a;
    float red, green, blue, alpha;
    unsigned char current_byte;
    char buffer[256];
    RGBAImageFileType file_type = RPPM;
    
    FILE *fptr=fopen(filename,"rb");
    if(fptr == NULL)
    {
	fprintf(stderr,"\nError: could not open %s on node\n",filename);
	return -1;
    }
    
    fgets(buffer,256,fptr);
    buffer[strlen(buffer)-1]='\0';
    //    fprintf(stderr,"\n%d Buffer %s|",strlen(buffer),buffer);
    
    if(strcmp(buffer, "P1") == 0)
	file_type = PBM;
    else if(strcmp(buffer, "P2") == 0)
	file_type = PGM;
    else if(strcmp(buffer, "P3") == 0)
	file_type = PPM;
    else if(strcmp(buffer, "P4") == 0)
	file_type = RPBM;
    else if(strcmp(buffer, "P5") == 0)
	file_type = RPGM;
    else if(strcmp(buffer, "P6") == 0)
	file_type = RPPM;
    else if(strcmp(buffer, "P7") == 0)
	file_type = PNMA;
    else if(strcmp(buffer, "P8") == 0)
	file_type = RPNMA;
    else 
    {
      fprintf(stderr,"\nError in Magic Number");
      return -1;
    }

//    fprintf(stderr,"\nAbout to discard comment line");
    while(discardCommentLine(fptr));
    //    discardCommentLine(fptr);
    
    fgets(buffer,255,fptr);
    //    fprintf(stderr,"\n%s",buffer);
    sscanf(buffer,"%d %d",&x,&y);
    
    if((file_type != PBM) && (file_type != RPBM) )
    {
	fgets(buffer,256,fptr);
	//	fprintf(stderr,"\n%s",buffer);
	
	sscanf(buffer,"%d",&depth);
	//	fprintf(stderr,"\n%d",depth);
    } 
    else 
      depth = 255;
    
    if(depth != 255)
    {
	fprintf(stderr,"Error: plane depth not equal to 255.");
  	return -1;	
    }
    
    if((x <= 0) || (y <= 0))
    {
	fprintf(stderr,"Invalid image size: %d %d\n",x,y);
  	return -1;	
    } 
    
    fread(rgb,sizeof(char),x*y*3,fptr);
    
    fclose(fptr);
//    fprintf(stderr,"About to return\n");
    
}// End of pnmLoader()

int 
pnmGetImageSize(char *filename,int &x,int &y)
{
    int depth, index;
    int r, g, b, a;
    float red, green, blue, alpha;
    unsigned char current_byte;
    char buffer[256];
    RGBAImageFileType file_type = RPPM;
    
    FILE *fptr=fopen(filename,"rb");
    if(fptr == NULL)
    {
	fprintf(stderr,"\nError: could not open %s on node\n",filename);
	return -1;
    }
    
    fgets(buffer,256,fptr);
//    fprintf(stderr,"\n%d Buffer %s|",strlen(buffer),buffer);
    
    if(strcmp(buffer, "P1") == 0)
	file_type = PBM;
    else if(strcmp(buffer, "P2") == 0)
	file_type = PGM;
    else if(strcmp(buffer, "P3") == 0)
	file_type = PPM;
    else if(strcmp(buffer, "P4") == 0)
	file_type = RPBM;
    else if(strcmp(buffer, "P5") == 0)
	file_type = RPGM;
    else if(strcmp(buffer, "P6") == 0)
	file_type = RPPM;
    else if(strcmp(buffer, "P7") == 0)
	file_type = PNMA;
    else if(strcmp(buffer, "P8") == 0)
	file_type = RPNMA;
    else 
    {
      //	fprintf(stderr,"\nError in Magic Number");
	//	return -1;
    }

//    fprintf(stderr,"\nAbout to discard comment line");
    while(discardCommentLine(fptr));
    
    fgets(buffer,256,fptr);
//    fprintf(stderr,"\n%s",buffer);
    sscanf(buffer,"%d %d",&x,&y);

    fclose(fptr);

}// End of pnmGetImageSize(char *,int, int);

int
pnmReverseImage(int x,int y,char *rgb)
{
    int i,j,k;
    j = 0;
    char *tmp = new char[x*y*3];
    memcpy(tmp,rgb,3*x*y*sizeof(char));
    for(i=y;i>=0;i-=3)
	for(j=0;j<x;j+=3)
	{
	    rgb[k] = tmp[i*x*3+j*3];
	    rgb[k+1] = tmp[i*x*3+j*3+1];
	    rgb[k+2] = tmp[i*x*3+j*3+2];
	    k = k+3;
	}
#if 0
    for(i=(3*x*y-1);i>= 0;i-=3)
    {
	rgb[j] = tmp[i-2];
	rgb[j+1] = tmp[i-1];
	rgb[j+2] = tmp[i];
	j += 3;
    }
#endif
    delete [] tmp;
}

void main(int argc,char *argv[]){
  struct{
    int w,h,numFrames;
  }tmpStr;
  FILE *outFile;

  count=0;
  if(argc<4){
    fprintf(stderr,"Need args!\n");
    exit(-1);
  };

  strcpy(filenamePre,argv[1]);
  if(argc<5){
    strcpy(filenamePost,"");
    numberOfFrames=atoi(argv[2]);
    frameOffset=atoi(argv[3]);
    strcpy(outFilename,argv[4]);
  }
  else{
    strcpy(filenamePost,argv[2]);
    numberOfFrames=atoi(argv[3]);
    frameOffset=atoi(argv[4]);
    strcpy(outFilename,argv[5]);
  };

  if((outFile=fopen(outFilename,"wb"))==NULL){
    fprintf(stderr,"ERROR: Could not open output file: %s\n",outFilename);
    exit(-1);
  };

  sprintf(filename1,"%s%.4d%s.pnm",filenamePre,count+frameOffset,filenamePost);
  pnmGetImageSize(filename1,x,y);
  rgba=new char[x*y*3];
  tmpStr.w=htonl(x);
  tmpStr.h=htonl(y);
  tmpStr.numFrames=htonl(numberOfFrames);
  fwrite(&tmpStr,sizeof(tmpStr),1,outFile);

  while(count<numberOfFrames){
    sprintf(filename1,"%s%.4d%s.pnm",filenamePre,count+frameOffset,filenamePost);
    pnmLoadImage(filename1,x,y,rgba);
    fwrite(rgba,x*y*3,1,outFile);
    count++;
  };

  fclose(outFile);
};





